import * as Lambda from 'aws-lambda';
import { OpenApi } from '../../openapi';
import { OperationParams, RawRequest, RawResponse, RequestParams } from '../../types';
/**
 * AWS Lambda specific request parameters
 * @property event    The Lambda event
 * @property context  The Lambda context
 */
export declare type LambdaSource = {
    lambda: {
        event: Lambda.APIGatewayEvent;
        context: Lambda.Context;
    };
};
export declare type LambdaOperationParams<T = any> = OperationParams<LambdaSource & T>;
export declare type LambdaRequestParams<T = any> = RequestParams<LambdaSource & T>;
/**
 * A HTTP API using an OpenAPI definition and implemented using AWS Lambda.
 */
export declare class LambdaOpenApi<T> extends OpenApi<LambdaSource & T> {
    protected fromLambdaEvent(event: Lambda.APIGatewayEvent): RawRequest;
    protected toLambdaResult(res: RawResponse): Lambda.APIGatewayProxyResult;
    /**
     * Creates a lambda HTTP event handler function which will route and handle requests using this class
     * and transform the response into a lambda HTTP event response.
     * The request and response body will be converted from/to JSON.
     *
     * @param args  If T is an object, an instance of T, otherwise no parameters
     *
     * @return A lambda event handler function
     */
    eventHandler(...[data]: T extends Record<string, unknown> ? T[] : []): Lambda.APIGatewayProxyHandler;
}
//# sourceMappingURL=lambda-openapi.d.ts.map