"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LambdaOpenApi = void 0;
const openapi_1 = require("../../openapi");
function parseJson(body) {
    // Try to parse the body as JSON. If it's malformed, we return the raw string as the body to get a useful
    // error message from the API validator.
    try {
        return body ? JSON.parse(body) : undefined;
    }
    catch (err) {
        return body;
    }
}
// https://github.com/DefinitelyTyped/DefinitelyTyped/pull/50224 ¯\_(ツ)_/¯
function trimRecord(obj) {
    return Object.fromEntries(Object.entries(obj).filter(([, v]) => v != null));
}
/**
 * A HTTP API using an OpenAPI definition and implemented using AWS Lambda.
 */
class LambdaOpenApi extends openapi_1.OpenApi {
    fromLambdaEvent(event) {
        return {
            method: event.httpMethod,
            path: event.path,
            query: trimRecord(event.queryStringParameters || {}),
            headers: trimRecord(Object.assign(Object.assign({}, event.headers), event.multiValueHeaders)),
            // //params: trimRecord(event.pathParameters || {}),
            body: parseJson(event.body),
        };
    }
    toLambdaResult(res) {
        const statusCode = res.statusCode;
        const headers = {};
        const multiValueHeaders = {};
        const body = JSON.stringify(res.body);
        // Lambda separates ordinary headers and multi value headers
        for (const [k, v] of Object.entries(res.headers)) {
            if (Array.isArray(v)) {
                multiValueHeaders[k] = v.map(x => x.toString());
            }
            else if (v !== undefined) {
                headers[k] = v.toString();
            }
        }
        return {
            statusCode,
            headers,
            multiValueHeaders,
            body,
        };
    }
    /**
     * Creates a lambda HTTP event handler function which will route and handle requests using this class
     * and transform the response into a lambda HTTP event response.
     * The request and response body will be converted from/to JSON.
     *
     * @param args  If T is an object, an instance of T, otherwise no parameters
     *
     * @return A lambda event handler function
     */
    eventHandler(...[data]) {
        return (event, context) => __awaiter(this, void 0, void 0, function* () {
            this.logger.debug(`Lambda event:\n${JSON.stringify(event, null, 2)}`);
            const res = yield this.handleRequest(this.fromLambdaEvent(event), Object.assign({ lambda: {
                    event,
                    context
                } }, data));
            return this.toLambdaResult(res);
        });
    }
}
exports.LambdaOpenApi = LambdaOpenApi;
//# sourceMappingURL=lambda-openapi.js.map